/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.flint.actions
{
	import flash.display.DisplayObject;
	
	import org.flintparticles.actions.Move;
	import org.flintparticles.emitters.Emitter;
	import org.flintparticles.particles.Particle;

	/**
	 * An extension of the Move action that allows you to specify that certain
	 * particles should not move.
	 * 
	 * @author Josh Tynjala
	 */
	public class MoveWithExceptions extends Move
	{
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 */
		public function MoveWithExceptions()
		{
			super();
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * @private
		 * Storage for the exception.
		 */
		protected var exceptions:Array = [];
		
	//----------------------------------
	//  Public Methods
	//----------------------------------
		
		/**
		 * Adds a particle's image as an exception.
		 */
		public function addException(image:DisplayObject):void
		{
			this.exceptions.push(image);
		}
		
		/**
		 * Removes an exception.
		 */
		public function removeException(image:DisplayObject):void
		{
			var index:int = this.exceptions.indexOf(image);
			if(index >= 0)
			{
				this.exceptions.splice(index, 1);
			}
		}
		
		/**
		 * @inheritDoc
		 */
		override public function update(emitter:Emitter, particle:Particle, time:Number):void
		{
			if(this.exceptions.indexOf(particle.image) >= 0)
			{
				return;
			}
			super.update(emitter, particle, time);
		}
		
	}
}