/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.flint.actions
{
	import org.flintparticles.actions.Action;
	import org.flintparticles.emitters.Emitter;
	import org.flintparticles.particles.Particle;

	/**
	 * Quickly fades in a particle's image when it first appears. The Age action
	 * is required for this to work. If you don't want the particles to die, set
	 * the Lifetime initializer to Number.POSITIVE_INFINITY.
	 * 
	 * @author Josh Tynjala
	 */
	public class QuickFade extends Action
	{
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 */
		public function QuickFade()
		{
			super();
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * The duration of the fade, in milliseconds.
		 */
		public var duration:Number = 250;
		
		/**
		 * The easing function used by the fade animation.
		 */
		public var easingFunction:Function = function(t:Number, b:Number, c:Number, d:Number):Number
		{
			return c * t / d + b;
		}
		
	//----------------------------------
	//  Public Methods
	//----------------------------------
		
		/**
		 * @inheritDoc
		 */
		override public function update(emitter:Emitter, particle:Particle, time:Number):void
		{
			var alpha:Number = easingFunction(particle.age * 1000, 0, 1, duration);
			alpha = Math.min(alpha, 1);
			particle.color = ( particle.color & 0xFFFFFF ) | ( Math.round( alpha * 255 ) << 24 );
		}
	}
}