/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.flint.counters
{
	import flash.display.DisplayObject;
	
	import mx.core.IFactory;
	
	import org.flintparticles.counters.Counter;
	import org.flintparticles.emitters.Emitter;

	/**
	 * A counter that adds particles only when new "data" is available.
	 * 
	 * @author Josh Tynjala
	 */
	public class DataCounter implements Counter
	{
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 */
		public function DataCounter()
		{
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * The renderer to which we're adding particles.
		 */
		public var targetRenderer:DisplayObject;
		
		/**
		 * A factory to create the image for the particle.
		 */
		public var displayObjectFactory:IFactory;
		
		/**
		 * The field on the particle image where the data is stored.
		 */
		public var dataField:String;
		
		/**
		 * The maximum number of particles to add per frame.
		 */
		public var maximumPerFrame:Number = Number.MAX_VALUE;
		
		/**
		 * @private
		 * Storage for the data that is yet to be added.
		 */
		protected var newData:Array = [];
		
		/**
		 * @private
		 * The number of particles to add in the current frame.
		 */
		protected var count:Number = 0;
		
	//----------------------------------
	//  Public Methods
	//----------------------------------
	
		/**
		 * Adds one item to the counter to be added
		 */
		public function addData(data:Object):void
		{
			this.newData.push(data);
		}

		/**
		 * @inheritDoc
		 */
		public function startEmitter(emitter:Emitter):uint
		{
			this.addDisplayObjects(emitter);
			return 0;
		}
		
		/**
		 * @inheritDoc
		 */
		public function updateEmitter(emitter:Emitter, time:Number):uint
		{
			this.addDisplayObjects(emitter);
			return 0;
		}
		
	//----------------------------------
	//  Protected Methods
	//----------------------------------
		
		/**
		 * @private
		 * Adds new display objects to the emitter as particles.
		 */
		protected function addDisplayObjects(emitter:Emitter):void
		{
			var dispObjsToAdd:Array = [];
			var newDataCount:Number = Math.min(this.maximumPerFrame, this.newData.length);
			count += newDataCount;
			if(count < 1)
			{
				return;
			}
			count--;
			
			for(var i:int = 0; i < newDataCount; i++)
			{
				var data:Object = this.newData.shift();
				var dispObj:DisplayObject = this.displayObjectFactory.newInstance();
				dispObj[this.dataField] = data;
				if(targetRenderer)
				{
					dispObj.x = (targetRenderer.width - dispObj.width) / 2;
					dispObj.y = (targetRenderer.height - dispObj.height) / 2;
				}
				dispObjsToAdd.push(dispObj);
			}
			
			emitter.addDisplayObjects(dispObjsToAdd);
		}
	}
}