/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.flint.renderers
{
	import mx.core.UIComponent;
	
	import org.flintparticles.particles.Particle;
	import org.flintparticles.renderers.Renderer;

	/**
	 * A Flint renderer for particle images that are Flex UIComponents.
	 * 
	 * @author Josh Tynjala
	 */
	public class UIComponentRenderer extends UIComponent implements Renderer
	{
		
	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 */
		public function UIComponentRenderer()
		{
			super();
			this.mouseEnabled = false;
			this.mouseChildren = false;
		}
		
	//----------------------------------
	//  Public Methods
	//----------------------------------
		
		/**
		 * @inheritDoc
		 */
		public function renderParticles(particles:Array):void
		{
			particles.forEach(function(particle:Particle, index:int, source:Array):void
			{
				particle.image.transform.colorTransform = particle.colorTransform;
				particle.image.transform.matrix = particle.matrixTransform;
			});
		}
		
		/**
		 * @inheritDoc
		 */
		public function addParticle( particle:Particle ):void
		{
			this.addChildAt(particle.image, 0);
		}
		
		/**
		 * @inheritDoc
		 */
		public function removeParticle( particle:Particle ):void
		{
			this.removeChild(particle.image);
		}
		
	//----------------------------------
	//  Protected Methods
	//----------------------------------
		
		/**
		 * @private
		 */
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			var childCount:int = this.numChildren;
			for(var i:int = 0; i < childCount; i++)
			{
				var child:UIComponent = this.getChildAt(i) as UIComponent;
				if(child)
				{
					child.setActualSize(child.getExplicitOrMeasuredWidth(), child.getExplicitOrMeasuredHeight());
				}
			}
		}
		
	}
}