/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.mate.actions.builders
{
	import com.asfusion.mate.actionLists.IScope;
	import com.asfusion.mate.actions.builders.ObjectBuilder;
	import com.asfusion.mate.core.*;
	import com.asfusion.mate.utils.debug.*;
	use namespace com.asfusion.mate.core.mate;

	/**
	 * Similar to MethodInvoker, but passes a value to a setter rather than a
	 * method.
	 * 
	 * @author Josh Tynjala
	 */
	public class PropertyInvoker extends ObjectBuilder
	{
		
	//----------------------------------
	//  Properties
	//----------------------------------
	
		/**
		 * @private
		 * Storage for the value property.
		 */
		private var _value:*;
		
		/**
		 * The new value for the property.
		 */
		public function get value():*
		{
			return this._value;
		}
		
		/**
		 * @private
		 */
		public function set value(value:*):void
		{
			this._value = value;
		}
		
		/**
		 * @private
		 * Storage for the property property.
		 */
		private var _property:String;
		
		/**
		 * The name of the property to change.
		 */
		public function get property():String
		{
			return this._property;
		}
		
		/**
		 * @private
		 */
		public function set property(value:String):void
		{
			this._property = value;
		}
		
	//----------------------------------
	//  Protected Methods
	//----------------------------------
		
		/**
		 * @inheritDoc
		 */
		override protected function run(scope:IScope):void
		{
			if(this.property)
			{
				var source:Object = {};
				source[this.property] = value;
				scope.lastReturn = Properties.smartCopy(source, currentInstance, scope);
			}
			else
			{
				var logInfo:LogInfo = new LogInfo( scope, currentInstance, null, null, this.value);
				scope.getLogger().error(LogTypes.SOURCE_UNDEFINED, logInfo);
			}
		}
	}
}