/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.roktok.business
{
	import com.flextoolbox.roktok.events.MessagingServiceEvent;
	import com.flextoolbox.roktok.vo.Artist;
	import com.flextoolbox.roktok.vo.Message;
	
	import flash.events.Event;
	import flash.events.EventDispatcher;
	import flash.events.IEventDispatcher;
	import flash.utils.setTimeout;
	
	import mx.collections.ArrayCollection;

	/**
	 * Manages the artist data.
	 * 
	 * @author Josh Tynjala
	 */
	public class ArtistsManager extends EventDispatcher
	{
		
	//----------------------------------
	//  Static Properties
	//----------------------------------
	
		/**
		 * @private
		 * Regular expression to remove punctuation from artist names.
		 */
		private static const PUNCTUATION:RegExp = /[-!"#$%&'()*+,.\/:;<=>?@[\\\]_`{|}~]/g;
		
	//----------------------------------
	//  Properties
	//----------------------------------
			
		/**
		 * @private
		 * Storage for the artists property.
		 */
		private var _artists:ArrayCollection = new ArrayCollection();
		
		[Bindable("artistsChange")]
		/**
		 * The artists that the application will visualize.
		 */
		public function get artists():ArrayCollection
		{
			return this._artists;
		}
		
	//----------------------------------
	//  Public Methods
	//----------------------------------
		
		/**
		 * Stores the artist data in the manager.
		 */
		public function storeArtists(value:Array):void
		{
			this._artists = new ArrayCollection(value);
			this.dispatchEvent(new Event("artistsChange"));			
		}
		
		/**
		 * Determines which artist each message is about.
		 * 
		 * @param messages		The messages to associate with artists.
		 */
		public function associateMessagesWithArtist(messages:Array):Array
		{
			var artists:Array = this.artists.source;
			for each(var artist:Artist in artists)
			{
				var artistName:String = artist.name.toLowerCase().replace(PUNCTUATION, "");
				//var artist:Artist = Artist(this.artists.getItemAt(this.searchIndex - 1));
				messages.forEach(function(message:Message, index:int, source:Array):void
				{
					if(message.text.toLowerCase().replace(PUNCTUATION, "").indexOf(artistName) >= 0)
					{
						message.artist = artist;
					}
				});
			}
			
			//if we couldn't find the artist, then remove the message
			messages = messages.filter(function(message:Message, index:int, source:Array):Boolean
			{
				return message.artist != null;
			});
			return messages;
		}
		
		/**
		 * Builds a Twitter search query for the artists stored in the manager. 
		 */
		public function buildSearchQuery():String
		{
			var query:String = "";
			
			var artists:Array = this.artists.source.concat();
			
			var artistCount:int = artists.length;
			for(var i:int = 0; i < artistCount; i++)
			{
				var artist:Artist = Artist(artists[i]);
				query += '"' + artist.name.toLowerCase().replace(PUNCTUATION, "") + '"';
				if(i < artistCount - 1)
				{
					query += " OR ";
				}	
			}
			return query;
		}
		
	}
}