/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.roktok.business
{
	import com.flextoolbox.roktok.vo.Artist;
	
	/**
	 * Parses responses from the Yahoo! Music API.
	 * 
	 * @author Josh Tynjala
	 */
	public class YahooMusicServiceParser
	{
		
	//----------------------------------
	//  Static Properties
	//----------------------------------
	
		/**
		 * A set of colors to associate with artists when they are parsed.
		 * 
		 * These are the same default colors that are used in the standard Flex charts.
		 */
		public static const ARTIST_COLORS:Array =
		[
			0xE48701, 0xA5BC4E, 0x1B95D9, 0xCACA9E, 0x6693B0,
			0xF05E27, 0x86D1E4, 0xE4F9A0, 0xFFD512, 0x75B000,
			0x0662B0, 0xEDE8C6, 0xCC3300, 0xD1DFE7, 0x52D4CA,
			0xC5E05D, 0xE7C174, 0xFFF797, 0xC5F68F, 0xBDF1E6,
			0x9E987D, 0xEB988D, 0x91C9E5, 0x93DC4A, 0xFFB900,
			0x9EBBCD, 0x009797, 0x0DB2C2
		];
		
	//----------------------------------
	//  Public Methods
	//----------------------------------
		
		/**
		 * Parses an XML response from the Yahoo! Music API and returns an Array of Artist VOs.
		 */
		public function parseListPopularArtistsResult(result:String):Array
		{
			var data:XML = new XML(result);
			
			var popularArtists:Array = [];
			for each(var artistData:XML in data.Artist)
			{
				var artist:Artist = new Artist();
				artist.name = artistData.@name.toString();
				artist.url = artistData.@website.toString();
				artist.currentChartPosition = artistData.ItemInfo.ChartPosition.attribute("this").toString();
				artist.lastChartPosition = artistData.ItemInfo.ChartPosition.@last.toString();
				artist.color = ARTIST_COLORS[popularArtists.length % ARTIST_COLORS.length];
				popularArtists.push(artist);
			}
			return popularArtists;
		}

	}
}