/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.roktok.views
{
	import com.flextoolbox.roktok.vo.Message;
	
	import flash.text.TextFieldAutoSize;
	
	import mx.core.UIComponent;
	import mx.core.UITextField;
	import mx.styles.CSSStyleDeclaration;
	import mx.styles.StyleManager;

	/**
	 * A data tip that is displayed when the mouse is over a message particle.
	 * 
	 * @author Josh Tynjala
	 */
	public class ParticleDataTip extends UIComponent
	{
		
	//----------------------------------
	//  Static Methods
	//----------------------------------
	
		/**
		 * @private
		 * Sets the default style values.
		 */
		private static function initStyles():void
		{
			var styles:CSSStyleDeclaration = StyleManager.getStyleDeclaration("ParticleDataTip");
			if(!styles)
			{
				styles = new CSSStyleDeclaration();
			}
				
			styles.defaultFactory = function():void
			{
				this.backgroundColor = 0x000000;
				this.backgroundAlpha = 0.7;
				this.paddingTop = 4;
				this.paddingRight = 4;
				this.paddingBottom = 4;
				this.paddingLeft = 4;
				this.cornerRadius = 0;
				this.color = 0xffffff;
			}
				
			StyleManager.setStyleDeclaration("ParticleDataTip", styles, true);
		}
		initStyles();

	//----------------------------------
	//  Constructor
	//----------------------------------
	
		/**
		 * Constructor.
		 */
		public function ParticleDataTip()
		{
			super();
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * @private
		 * The TextField that displays information about the message.
		 */
		protected var messageField:UITextField;
		
		/**
		 * @private
		 * Storage for the message property.
		 */
		private var _message:Message;
		
		/**
		 * The message with which the particle is associated.
		 */
		public function get message():Message
		{
			return this._message;
		}
		
		/**
		 * @private
		 */
		public function set message(value:Message):void
		{
			this._message = value;
			this.invalidateProperties();
			this.invalidateSize();
			this.invalidateDisplayList();
		}
		
	//----------------------------------
	//  Protected Methods
	//----------------------------------
	
		/**
		 * @private
		 */
		override protected function createChildren():void
		{
			super.createChildren();
			
			if(!this.messageField)
			{
				this.messageField = new UITextField();
				this.messageField.styleName = this;
				this.messageField.multiline = true;
				this.messageField.autoSize = TextFieldAutoSize.LEFT;
				this.messageField.mouseEnabled = false;
				this.messageField.selectable = false;
				this.addChild(this.messageField);
			}
		}
		
		/**
		 * @private
		 */
		override protected function commitProperties():void
		{
			super.commitProperties();
			
			if(this.message)
			{
				this.messageField.htmlText = "<b><font color=\"#03bd00\">" + this.message.user + "</font></b><br>talks about <b><font color=\"#" + this.message.artist.color.toString(16) + "\">" + this.message.artist.name + "</font></b>";
			}
			else
			{
				this.messageField.text = "";
			}
		}
		
		/**
		 * @private
		 */
		override protected function measure():void
		{
			super.measure();
			
			var paddingTop:Number = this.getStyle("paddingTop");
			var paddingRight:Number = this.getStyle("paddingRight");
			var paddingBottom:Number = this.getStyle("paddingBottom");
			var paddingLeft:Number = this.getStyle("paddingLeft");
			this.measuredWidth = this.messageField.measuredWidth + paddingLeft + paddingRight;
			this.measuredHeight = this.messageField.measuredHeight + paddingTop + paddingBottom;
		}
		
		/**
		 * @private
		 */
		override protected function updateDisplayList(unscaledWidth:Number, unscaledHeight:Number):void
		{
			super.updateDisplayList(unscaledWidth, unscaledHeight);
			
			var backgroundColor:uint = this.getStyle("backgroundColor");
			var backgroundAlpha:Number = this.getStyle("backgroundAlpha");
			var cornerRadius:Number = this.getStyle("cornerRadius");
			
			this.graphics.clear();
			this.graphics.beginFill(backgroundColor, backgroundAlpha);
			this.graphics.drawRoundRect(0, 0, unscaledWidth, unscaledHeight, cornerRadius, cornerRadius);
			this.graphics.endFill();
			
			var paddingTop:Number = this.getStyle("paddingTop");
			var paddingRight:Number = this.getStyle("paddingRight");
			var paddingBottom:Number = this.getStyle("paddingBottom");
			var paddingLeft:Number = this.getStyle("paddingLeft");
			
			this.messageField.x = paddingLeft;
			this.messageField.y = paddingTop;
			this.messageField.setActualSize(unscaledWidth - paddingLeft - paddingRight, unscaledHeight - paddingTop - paddingBottom);
		}
		
	}
}