/*

Copyright (c) 2008 Esria, Inc. (esria.com)

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

*/
package com.flextoolbox.roktok.views
{
	import flash.display.DisplayObject;
	import flash.display.Shape;
	import flash.display.Sprite;
	import flash.events.Event;
	import flash.events.ProgressEvent;
	import flash.events.TimerEvent;
	import flash.utils.Timer;
	import flash.utils.setTimeout;
	
	import mx.events.FlexEvent;
	import mx.preloaders.DownloadProgressBar;

	/**
	 * Custom preloader. We don't want to see that silly Flex default, do we?
	 * 
	 * @author Josh Tynjala
	 */
	public class RokTokPreloader extends DownloadProgressBar
	{
		
	//----------------------------------
	//  Static Properties
	//----------------------------------
	
		[Embed(source="/assets/preloader_logo.png")]
		/**
		 * @private
		 * Embedded logo. I don't want to embed the font, but I want to fade in
		 * some text. Flash Player 10, please come out soon!
		 */
		private static const ROKTOK_LOGO:Class;
		
	//----------------------------------
	//  Constructor
	//----------------------------------
		
		/**
		 * Constructor.
		 */
		public function RokTokPreloader()
		{
			super();
			this.addEventListener(Event.ADDED_TO_STAGE, addedToStageHandler);
			
			this._progressBar = new Shape();
			this.addChild(this._progressBar);
			
			this._logo = new ROKTOK_LOGO();
			this._logo.alpha = 0;
			this.addChild(this._logo);
			
			this._logoTimer = new Timer(10);
			this._logoTimer.addEventListener(TimerEvent.TIMER, timerUpdateHandler);
			this._logoTimer.start();
		}
		
	//----------------------------------
	//  Properties
	//----------------------------------
		
		/**
		 * @private
		 * A simple progress bar to track the download.
		 */
		private var _progressBar:Shape;
		
		/**
		 * @private
		 * RokTok logo.
		 */
		private var _logo:DisplayObject;
		
		/**
		 * @private
		 * A timer used for animations.
		 */
		private var _logoTimer:Timer;
		
		/**
		 * @private
		 * Flag used to track whether the animation and the load are both complete.
		 */
		private var _otherIsDone:Boolean = false;
		
		/**
		 * @private
		 */
		override public function set preloader( preloader:Sprite ):void 
		{				   
		    preloader.addEventListener( ProgressEvent.PROGRESS , swfProgressHandler );
		    preloader.addEventListener( FlexEvent.INIT_COMPLETE , initCompleteHandler );
		}
		
	//----------------------------------
	//  Private Methods
	//----------------------------------
		
		/**
		 * @private
		 * Position the logo and draw the progress bar. 
		 */
		private function redraw():void
		{	
			this._logo.x = (this.stage.stageWidth - this._logo.width) / 2;
			this._logo.y = (this.stage.stageHeight - this._logo.height) / 2;
			
			var percentageWidth:Number = this.stage.stageWidth * this.loaderInfo.bytesLoaded / this.loaderInfo.bytesTotal;
			
			this._progressBar.graphics.clear();
			this._progressBar.graphics.beginFill(0xffffff);
			this._progressBar.graphics.drawRect(0, this.stage.stageHeight - 4, percentageWidth, 4);
			this._progressBar.graphics.endFill();
		}
		
		/**
		 * @private
		 * Fade out the logo. 
		 */
		private function preloadComplete():void
		{
			if(!this._otherIsDone)
			{
				this._otherIsDone = true;
				return;
			}
    		this._logoTimer.addEventListener(TimerEvent.TIMER, timerUpdateHandler2);
    		this._logoTimer.start();
			
		}
		
	//----------------------------------
	//  Private Event Handlers
	//----------------------------------
		
		/**
		 * @private
		 * Listen for the stage resize so that we can reposition the logo.
		 */
		private function addedToStageHandler(event:Event):void
		{
			this.removeEventListener(Event.ADDED_TO_STAGE, addedToStageHandler);
			this.stage.addEventListener(Event.RESIZE, stageResizeHandler, false, 0, true);
		}
		
		/**
		 * @private
		 * Redraw the progress bar when we get a new progress indicator. 
		 */
		private function swfProgressHandler(event:ProgressEvent):void
		{
			this.redraw();
		}
		
		/**
		 * @private
		 * Redraw when the stage resizes.
		 */
		private function stageResizeHandler(event:Event):void
		{
			this.redraw();
		}
		
		/**
		 * @private
		 * App is ready, let's go!
		 */
		private function initCompleteHandler(event:FlexEvent):void
		{
			this.preloadComplete();
		}
		
		/**
		 * @private
		 * Handles fading in the logo.
		 */
		private function timerUpdateHandler(event:TimerEvent):void
		{
			var timeElapsed:Number = this._logoTimer.currentCount * 10; //roughly...
			var newAlpha:Number = Math.min(timeElapsed / 800, 1);
			this._logo.alpha = newAlpha;
			
			if(newAlpha == 1)
			{
				this._logoTimer.removeEventListener(TimerEvent.TIMER, timerUpdateHandler);
				this._logoTimer.stop();
				this._logoTimer.reset();
				this.preloadComplete();
			}
		}
		
		/**
		 * @private
		 * Handles fading out the logo.
		 */
		private function timerUpdateHandler2(event:TimerEvent):void
		{
			var timeElapsed:Number = this._logoTimer.currentCount * 10;
			var newAlpha:Number = Math.max((350 - timeElapsed) / 350, 0);
			this._logo.alpha = newAlpha;
			
			if(newAlpha == 0)
			{
				this._logoTimer.stop();
				this._logoTimer.removeEventListener(TimerEvent.TIMER, timerUpdateHandler2);
				this.stage.removeEventListener(Event.RESIZE, stageResizeHandler);
				this.dispatchEvent(new Event(Event.COMPLETE));
			}
		}

		
	}
}